package top.ninwoo;

import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.context.junit4.SpringRunner;
import top.ninwoo.bishe.starter.entity.ContainerMonitor;
import top.ninwoo.bishe.starter.service.ClusterService;
import top.ninwoo.bishe.starter.service.NetworkService;
import top.ninwoo.client.BisheTestMain;
import top.ninwoo.common.entity.*;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

@RunWith(SpringRunner.class)
@SpringBootTest(classes = BisheTestMain.class)
public class YmwangDfsTests {

    @Autowired
    private ClusterService clusterService;

    @Resource
    private NetworkService networkService;


    @Test
    public void testSendSeparateConfig() {
        ArrayList<SeparatedClusterConfig> clusterConfigs = new ArrayList<>();
        SeparatedClusterConfig separatedClusterConfig = new SeparatedClusterConfig();
        // TODO: 这个ID应该是从借口获取的
        separatedClusterConfig.setEdgeNodeId("192.168.31.198:8881");
        ClusterConfig clusterConfigStore = new ClusterConfig();
        clusterConfigStore.setId(11112l);
        clusterConfigStore.setOwner("joliu");

        List<ContainerDescription> cds = new ArrayList<>();

        /*通过镜像"ymwang/dfs_transfer_client"启动一个名为client的容器，来作为文件的收发端（图片的切割、合并）*/
        ContainerDescription containerDescriptionClient = new ContainerDescription();
        containerDescriptionClient.setMode("normal");
        containerDescriptionClient.setReplicas(1);
        DockerContainer container = new DockerContainer();
        container.setName("dfs_client");//创建一个client容器
        container.setCommand("bash");
        container.setImage("dfs_client");
        container.setPorts("8099");//指定容器端口
        containerDescriptionClient.setDockerContainer(container);
        cds.add(containerDescriptionClient);//将client容器添加到list集合中

        /*通过镜像"ymwang/dfs_transfer_server"启动一个名为server1的容器，用于存储图片切片*/
        ContainerDescription containerDescriptionServer1 = new ContainerDescription();
        containerDescriptionServer1.setMode("normal");
        containerDescriptionServer1.setReplicas(20);
        DockerContainer containerServer1 = new DockerContainer();
        containerServer1.setName("dfs_server");//创建一个server1容器
        containerServer1.setCommand("sh");
        containerServer1.setImage("dfs_server");
        containerDescriptionServer1.setDockerContainer(containerServer1);
        cds.add(containerDescriptionServer1);//将server1容器添加到list集合中

        clusterConfigStore.setDockers(cds);

        NetworkTopology topo = new NetworkTopology();
        //topo.setAppNames(new String[]{"Run", "APP", "br:ovs1"});
        topo.setAppNames(new String[]{"dfs_client", "dfs_server", "br:ovs1"});
        // 这个参数好像没啥用
        topo.setTopologyId(11);
        topo.setTopology(new int[][]{{0,0,0},{0,0,0},{1,1,0}});
        clusterConfigStore.setTopology(topo);
        separatedClusterConfig.setClusterConfig(clusterConfigStore);
        clusterConfigs.add(separatedClusterConfig);

        clusterService.sendClusterConfigToEdgeNode(clusterConfigs);
    }

    //测试容器ip
    @Test
    public void getIpListByAppNameTest() {
        List<String> ipList = networkService.getIpListByAppName(11112L, "dfs_server");
        System.out.println(ipList);
    }

    @Test
    public void getIpList() {
        List<String> ipListS = networkService.getIpListByAppName(11112L, "dfs_server");
        String[] ipList = new String[ipListS.size()];
        for (int i = 0; i < ipListS.size(); i++) {
            if(!ipListS.isEmpty()){
                String ip_tmp = ipListS.get(i);
                String[] split_list = ip_tmp.split("/");
                ipList[i] = split_list[0];
            }
        }
        for (int j = 0; j < ipList.length; j++) {
            System.out.println(ipList[j]);
        }
    }


    @Test
    public void removeClusterFromEdgeNodeTest() {
        clusterService.removeClusterFromEdgeNode(11112L);
    }

    @Test
    public void getLogicalNetworkTopologyTest() {
        NetworkTopology logicalNetworkTopology = networkService.getLogicalNetworkTopology(11112L);
        System.out.println(logicalNetworkTopology);
    }


    @Test
    public void enableNetworkMonitorTest() throws InterruptedException {
        String res = networkService.enableNetworkMonitor(11112L, "dfs_server");
        System.out.println(res);
    }

    @Test
    public void cancelNetworkMonitorTest() {
        String run = networkService.cancelNetworkMonitor(11112L, "dfs_server");
        System.out.println(run);
    }

    @Test
    public void getNetworkMonitorTest() throws InterruptedException {
        List<ContainerMonitor> run = networkService.getContainerMonitors(11112L, "Run");
        int i = 0;
        while(i < 10) {
            run.forEach(c -> {
                System.out.println(c.getContainerInfo());
            });
            Thread.sleep(500);
            i++;
        }
    }
}

